declare module 'loopback-connector' {
    import { Callback, Connector as ConnectorInterface, DataSource, PromiseOrVoid, Transaction as TransactionInterface } from 'loopback-datasource-juggler';
    class Connector implements ConnectorInterface {
        name: string;
        dataSource?: DataSource;
        connect(callback?: Callback): PromiseOrVoid;
        disconnect(callback?: Callback): PromiseOrVoid;
        ping(callback?: Callback): PromiseOrVoid;
        execute?(...args: any[]): Promise<any>;
    }
    class ParameterizedSQL {
        merge(sql: string): ParameterizedSQL;
    }
    class SQLConnector extends Connector {
        constructor(name: string, settings?: any);
        execute(...args: any[]): Promise<any>;
        executeSQL(sql: string, params?: any[], options?: object, callback?: Callback): PromiseOrVoid;
        automigrate(models?: string | string[] | Callback, cb?: Callback): PromiseOrVoid;
        serializeObject(val: any): string;
    }
    class Transaction implements TransactionInterface {
        connector?: Connector;
        connection?: Object;
        constructor(connector: Connector, connection: Object);
        /**
         * Commit the transaction
         * @param callback
         */
        commit(callback?: Callback): PromiseOrVoid;
        /**
         * Rollback the transaction
         * @param callback
         */
        rollback(callback?: Callback): PromiseOrVoid;
        static begin(connector: Connector, options: Object, callback?: Callback): PromiseOrVoid<Transaction>;
    }
}
