import { Connector, ModelDefinition } from '@loopback/repository';
import { SqlConnectionPool, SqlDatabase, SqlRunResult, MetaModel } from 'sqlite3orm';
import { Sqlite3AllSettings, Sqlite3Settings } from './sqlite3-settings';
import { MetaModelFactory } from './meta-model-factory';
export declare const SQLITE3_CONNECTOR_NAME = "sqlite3x";
export declare class Sqlite3Connector implements Connector {
    name: string;
    readonly settings: Sqlite3AllSettings;
    readonly pool: SqlConnectionPool;
    readonly metaModels: MetaModelFactory;
    /* istanbul ignore next */
    static debugEnabled: boolean;
    constructor(settings?: Sqlite3Settings | Object);
    static debug(arg: any, ...args: any[]): void;
    /*************************************************************************************
     * Connector interface
     */
    /**
     * connect to Sqlite3 database pool
     *
     * @returns a void-promise
     */
    connect(): Promise<void>;
    /**
     * disconnect from Sqlite3 database pool
     *
     * @returns a void-promise
     */
    disconnect(): Promise<void>;
    /**
     * ping the database backend
     *
     * @returns a void-promise
     */
    ping(): Promise<void>;
    /**
     * get connecton from pool
     */
    getConnection(): Promise<SqlDatabase>;
    /**
     * begin transaction
     */
    begin(): Promise<SqlDatabase>;
    /**
     * execute sql on new connection from pool
     * release connection afterwards
     */
    execSql(sql: string, params?: any[]): Promise<any[] | SqlRunResult>;
    /**
     * commit transaction
     *
     * @param connection
     */
    commit(connection: SqlDatabase): Promise<void>;
    /**
     * rollback transaction
     *
     * @param connection
     */
    rollback(connection: SqlDatabase): Promise<void>;
    static runDQL(conn: SqlDatabase, sql: string, params?: any[]): Promise<any[]>;
    static runDML(conn: SqlDatabase, sql: string, params?: any[]): Promise<SqlRunResult>;
    runSQL(conn: SqlDatabase, sql: string, params?: any[]): Promise<any[] | SqlRunResult>;
    getMetaModel(modelName: string, lbModelDef: ModelDefinition, recreate?: boolean): MetaModel;
    getMetaModelFromJuggler(modelName: string, lbModelDef: any, recreate?: boolean): MetaModel;
    destroyMetaModel(modelName: string): void;
    destroyAllMetaModels(): void;
    dbName(modelOrProperty?: string): string | undefined;
    static enrichInputSettings(inputSettings: Sqlite3Settings | Object): Sqlite3AllSettings;
}
